@@ -8,24 +8,11 @@ import pydecode.constraints as cons
 import pydecode.optimization as opt
 import itertools
 from collections import defaultdict
-
-def get_all_hyperpaths(h):
-    "Get all possible hyperpaths."
-    def paths(node):
-        if node.is_terminal:
-            yield tuple()
-        else:
-            for edge in node.edges:
-                t = [paths(node) for node in edge.tail]
-                for below in itertools.product(*t):
-                    yield (edge,) + sum(below, ())
-    paths = [ph.Path(h, list(edges)) for edges in paths(h.root)]
-    return paths
-
+import pydecode.utils as utils
 
 def test_all():
     s = simple_hypergraph()
-    for path in get_all_hyperpaths(s):
+    for path in utils.all_paths(s):
         valid_path(s, path)
 
 def chain_hypergraph(size=100):
@@ -77,18 +64,17 @@ def random_hypergraph(size=50):
     assert len(hypergraph.edges) > 0
     return hypergraph
 
+
 def hypergraphs():
     for i in range(10):
         h = random_hypergraph()
-        print h
         yield h
     h = simple_hypergraph()
-    print h
     yield h
 
-
 # TESTS FOR HYPERGRAPH CONSTRUCTION.
 
+
 def test_numbering():
     """
     Check that the numbering nodes and edges is correct.
@@ -163,36 +149,18 @@ def valid_path(hypergraph, path):
 def test_construction():
     h = random_hypergraph()
 
-def rand_gen(arg=None):
-    return random.random()
-
-def random_inside_potentials(hypergraph):
-    return ph.InsidePotentials(hypergraph).build(rand_gen)
-
-def random_viterbi_potentials(hypergraph):
-    return ph.ViterbiPotentials(hypergraph).build(rand_gen)
-
-def random_log_viterbi_potentials(hypergraph):
-    return ph.LogViterbiPotentials(hypergraph).build(rand_gen)
-
-def rand_bool_gen(arg=None):
-    return random.random() > 0.5
-
-def random_bool_potentials(hypergraph):
-    return ph.BoolPotentials(hypergraph).build(rand_bool_gen)
-
 
 def test_best_path():
     """
     Test viterbi path finding.
     """
     for h in hypergraphs():
-        w = random_viterbi_potentials(h)
+        w = utils.random_log_viterbi_potentials(h)
         path = ph.best_path(h, w)
         nt.assert_not_equal(w.dot(path), 0.0)
         valid_path(h, path)
         same = False
-        for other_path in get_all_hyperpaths(h):
+        for other_path in utils.all_paths(h):
             assert w.dot(path) >= w.dot(other_path)
             if path == other_path: same = True
         assert same
@@ -203,7 +171,7 @@ def test_inside():
     Test inside chart gen.
     """
     for h in hypergraphs():
-        w = random_inside_potentials(h)
+        w = utils.random_inside_potentials(h)
         inside = ph.inside(h, w)
 
 
@@ -212,7 +180,7 @@ def test_outside():
     Test outside chart properties.
     """
     for h in hypergraphs():
-        w = random_viterbi_potentials(h)
+        w = utils.random_viterbi_potentials(h)
         path = ph.best_path(h, w)
         chart = ph.inside_values(h, w)
         best = w.dot(path)
@@ -220,21 +188,21 @@ def test_outside():
         out_chart = ph.outside_values(h, w, chart)
         for node in h.nodes:
             other = chart[node] * out_chart[node]
-            nt.assert_less_equal(other.value, best + 1e-4)
+            nt.assert_less_equal(other, best + 1e-4)
         for edge in path.edges:
             for node in edge.tail:
                 if node.is_terminal:
-                    nt.assert_almost_equal(other.value, best)
+                    nt.assert_almost_equal(other, best)
 
 
 def test_posteriors():
     "Check the posteriors by enumeration."
     for h in hypergraphs():
-        w = random_inside_potentials(h)
+        w = utils.random_inside_potentials(h)
         marg = ph.compute_marginals(h, w)
 
 
-        paths = get_all_hyperpaths(h)
+        paths = utils.all_paths(h)
         m = defaultdict(lambda: 0.0)
         total_score = 0.0
         for path in paths:
@@ -245,43 +213,43 @@ def test_posteriors():
 
         for edge in h.edges:
             nt.assert_almost_equal(
-                marg[edge].value / marg[h.root].value,
+                marg[edge] / marg[h.root],
                 m[edge.id] / total_score, places=4)
 
         chart = ph.inside(h, w)
-        nt.assert_almost_equal(chart[h.root].value, total_score, places=4)
+        nt.assert_almost_equal(chart[h.root], total_score, places=4)
 
 def test_max_marginals():
     """
     Test that max-marginals are correct.
     """
     for h in hypergraphs():
-        w = random_viterbi_potentials(h)
+        w = utils.random_viterbi_potentials(h)
         print w.show(h)
 
         path = ph.best_path(h, w)
         best = w.dot(path)
         print "BEST"
 
-        print "\n".join(["%20s : %s"%(h.label(edge), w[edge]) for edge in path.edges])
+        print "\n".join(["%20s : %s"%(edge.label, w[edge]) for edge in path.edges])
         print best
         nt.assert_not_equal(best, 0.0)
         max_marginals = ph.compute_marginals(h, w)
         for node in h.nodes:
             other = max_marginals[node]
-            nt.assert_less_equal(other.value, best + 1e-4)
+            nt.assert_less_equal(other, best + 1e-4)
 
         for edge in h.edges:
             other = max_marginals[edge]
-            nt.assert_less_equal(other.value, best + 1e-4)
+            nt.assert_less_equal(other, best + 1e-4)
             if edge in path:
-                nt.assert_almost_equal(other.value, best)
+                nt.assert_almost_equal(other, best)
 
 ### PRUNING CODE
 
 def test_pruning():
     for h in hypergraphs():
-        w = random_viterbi_potentials(h)
+        w = utils.random_viterbi_potentials(h)
 
         original_path = ph.best_path(h, w)
         new_hyper, new_potentials = ph.prune_hypergraph(h, w, -0.99)
@@ -298,25 +266,25 @@ def test_pruning():
                                new_potentials.dot(prune_path))
 
         # Test pruning amount.
-        prune = random.random()
+        prune = 0.001
         max_marginals = ph.compute_marginals(h, w)
-        new_hyper, new_potentials = ph.prune_hypergraph(h, w, 0.0)
+        new_hyper, new_potentials = ph.prune_hypergraph(h, w, prune)
 
         assert (len(new_hyper.edges) > 0)
         original_edges = {}
         for edge in h.edges:
-            original_edges[h.label(edge)] = edge
+            original_edges[edge.label] = edge
 
         new_edges = {}
         for edge in new_hyper.edges:
-            new_edges[h.label(edge)] = edge
+            new_edges[edge.label] = edge
 
         for name, edge in new_edges.iteritems():
 
             orig = original_edges[name]
             nt.assert_almost_equal(w[orig], new_potentials[edge])
             m = max_marginals[orig]
-            nt.assert_greater(m, prune * original_score)
+            nt.assert_greater(m, prune)
 
 
 ### CONSTRAINT CODE
@@ -325,7 +293,7 @@ def random_have_constraint(hypergraph):
     edge, = random.sample(hypergraph.edges, 1)
 
     def build_constraints(label):
-        l = hypergraph.label(edge)
+        l = edge.label
         if label == l:
             return [("have", 1)]
         return []
@@ -333,14 +301,12 @@ def random_have_constraint(hypergraph):
         build_constraints)
     return constraints, edge
 
-
-
 def random_constraint(hypergraph):
     "Produce a random constraint on an edge."
 
     edge, = random.sample(hypergraph.edges, 1)
     print edge.id
-    l = hypergraph.label(edge)
+    l = edge.label
 
     def build_constraints(label):
         if label == l:
@@ -356,7 +322,7 @@ def test_constraint():
     Test constraint checking.
     """
     for h in hypergraphs():
-        w = random_viterbi_potentials(h)
+        w = utils.random_viterbi_potentials(h)
         constraints, edge = random_constraint(h)
         path = ph.best_path(h, w)
         match = constraints.check(path)
@@ -375,7 +341,7 @@ def test_constraint():
 
 def test_subgradient():
     for h in hypergraphs():
-        w = random_log_viterbi_potentials(h)
+        w = utils.random_log_viterbi_potentials(h)
         constraints, edge = random_have_constraint(h)
         path = ph.best_path(h, w)
         match = constraints.check(path)
@@ -391,15 +357,19 @@ def test_subgradient():
 def test_lp():
     import pydecode.lp as lp
     for h in hypergraphs():
-        w = random_log_viterbi_potentials(h)
+        w = utils.random_log_viterbi_potentials(h)
+
         g = lp.HypergraphLP.make_lp(h, w)
         g.solve()
         path = g.path
         opath = ph.best_path(h, w)
-        nt.assert_almost_equal(w.dot(path), w.dot(opath))
 
+        nt.assert_almost_equal(w.dot(path), w.dot(opath))
         for edge in path.edges:
-            assert(edge in opath)
+            assert edge in opath
+
+
+        # Constraint.
         constraints, edge = random_have_constraint(h)
         g = lp.HypergraphLP.make_lp(h, w)
         g.add_constraints(constraints)
@@ -432,6 +402,12 @@ def test_semirings():
         potentials = ph.Inside.Potentials(hypergraph).build(lambda l: 0.5)
 
 
+def test_serialization():
+    for hypergraph in hypergraphs():
+        s = utils.hypergraph_to_json(hypergraph)
+        hyper2 = utils.json_to_hypergraph(s)
+        nt.assert_equal(len(hypergraph.edges), len(hyper2.edges))
+        nt.assert_equal(len(hypergraph.nodes), len(hyper2.nodes))
 ## CONSTRUCTION CODE
 
 @nt.raises(Exception)
@@ -472,4 +448,4 @@ def test_bad_constraints():
 
 
 if __name__ == "__main__":
-    test_subgradient()
+    test_serialization()
