@@ -105,7 +105,7 @@ class DynamicProgrammingModel(StructuredModel):
             constraints = self.constraints(x, hypergraph)
             hyperlp = lp.HypergraphLP.make_lp(hypergraph, potentials, integral=True)
             hyperlp.add_constraints(constraints)
-            hyperlp.pulp.solvers.GLPK(mip=1)
+            hyperlp.solve(pulp.solvers.GLPK(mip=1))
             path = hyperlp.path
         y = set()
         for edge in path:
